
import java.io.IOException;
import java.io.PrintWriter;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import uws.UWSException;
import uws.job.ErrorType;
import uws.job.JobDestructionPolicy;
import uws.job.JobList;
import uws.job.JobThread;
import uws.job.Result;
import uws.job.UWSJob;
import uws.job.parameters.DestructionTimeController.DateField;
import uws.job.parameters.InputParamController;
import uws.job.user.JobOwner;
import uws.service.UWSServlet;
import uws.service.UWSUrl;

public class TestServlet extends UWSServlet {
	private static final long serialVersionUID = 1L;

	private final static String HOME_PAGE = "WEB-INF/home.html";

	/** Default value for the job parameter named "time". (in seconds) */
	private final static long DEFAULT_TIME = 60;
	/** Maximum value for the job parameter named "time". (in seconds) */
	private final static long MAX_TIME = 360;

	/* Default duration between the creation and the destruction of a job. */
	private final static int DEFAULT_DESTRUCTION_DURATION = 90;
	/* Unit of the default duration between the creation and the destruction of a job. */
	private final static DateField DEFAULT_DESTRUCTION_DURATION_UNIT = DateField.SECOND;
	/* Maximum duration between the creation and the destruction of a job. */
	private final static int MAX_DESTRUCTION_DURATION = 10;
	/* Unit of the maximum duration between the creation and the destruction of a job. */
	private final static DateField MAX_DESTRUCTION_DURATION_UNIT = DateField.MINUTE;

	@Override
	public JobThread createJobThread(UWSJob jobDescription) throws UWSException{
		return new TestJob(jobDescription);
	}

	@Override
	public void initUWS() throws UWSException{
		// [OPTIONAL]
		// Let's configure the UWS service so that being able to deal with the parameter "time" for jobs:
		// Default value = 1 minute.
		// Maximum value = 1 hour.
		addExpectedAdditionalParameter("time");
		setInputParamController("time", new InputParamController(){
			@Override
			public Object getDefault(){
				return new Long(DEFAULT_TIME);
			}

			@Override
			public Object check(Object value) throws UWSException{
				if (value == null)
					return getDefault();

				long duration = (Long)getDefault();
				try{
					duration = Long.parseLong(value.toString());
				}catch(NumberFormatException nfe){
					throw new UWSException(UWSException.BAD_REQUEST, "Wrong value type for the parameter \"time\"! It must be an integer or long value.");
				}

				return (duration > MAX_TIME) ? MAX_TIME : duration;
			}

			@Override
			public boolean allowModification(){
				return true;
			}
		});

		// Create one job list named "jobs" (=> url: http://localhost:8080/uws1.1/jobs):
		JobList jl = new JobList("jobs");
		addJobList(jl);

		/* ******************************** */
		/* CONFIGURE JOB DESTRUCTION POLICY */
		/*    (3 policies are proposed)     */
		/* ******************************** */
		// POLICY 1 (default in UWS 1.1): Jobs are put in ARCHIVED phase only when the destruction time is reached:
		jl.setDestructionPolicy(JobDestructionPolicy.ARCHIVE_ON_DATE);
		// POLICY 2: Jobs are archived at automatic destruction but also when requested by the UWS user (?ACTION=DELETE):
		//jl.setDestructionPolicy(JobDestructionPolicy.ALWAYS_ARCHIVE);
		// POLICY 3 (default in UWS 1.0): Jobs are never put in ARCHIVED phase ; automatic destruction and user requested destruction are real job destruction:
		//jl.setDestructionPolicy(JobDestructionPolicy.ALWAYS_DELETE);

		// [OPTIONAL]
		// In order to test the automatic destruction, configure the default and maximum retention period:
		configureDestruction(DEFAULT_DESTRUCTION_DURATION, DEFAULT_DESTRUCTION_DURATION_UNIT, MAX_DESTRUCTION_DURATION, MAX_DESTRUCTION_DURATION_UNIT, true);

		/* ********************************** */
		/*  CONFIGURE THE BLOCKING BEHAVIOR   */
		/*  (2 already implemented solutions) */
		/* ********************************** */

		// POLICY 1: Block WAIT request at most 1 minute (default)
		setWaitPolicy(new uws.service.wait.LimitedBlockingPolicy(60));

		// POLICY 2: Block WAIT request at most 1 hour (1st parameter)
		//           AND limit blocking requests by user and job to 3 maximum (2nd parameter)
		//           ; when there is a new incoming request, the oldest one is stopped (3rd parameter):
		//setWaitPolicy(new uws.service.wait.UserLimitedBlockingPolicy(360, 3, true));
	}

	@Override
	protected void writeHomePage(UWSUrl requestUrl, HttpServletRequest req, HttpServletResponse resp, JobOwner user) throws UWSException, ServletException, IOException{
		if (req.getServletContext().getResource(HOME_PAGE) != null)
			req.getRequestDispatcher(HOME_PAGE).forward(req, resp);
		else
			super.writeHomePage(requestUrl, req, resp, user);
	}

	/**
	 * Job which is sleeping for a specified time.
	 */
	public static class TestJob extends JobThread {

		public TestJob(UWSJob j) throws NullPointerException{
			super(j);
		}

		@Override
		protected void jobWork() throws UWSException, InterruptedException{
			long duration = (Long)job.getAdditionalParameterValue("time");
			for(int i = 0; i < duration; i++){
				if (isInterrupted())
					throw new InterruptedException();
				Thread.sleep(1000);
			}

			/* 2. WRITE THE RESULT FILE */
			// Create the result:
			Result result = createResult();
			String content = duration + " seconds elapsed";

			// Set its MIME type:
			result.setMimeType("text/plain");

			PrintWriter writer = null;
			try{
				// Write the result file:
				writer = new PrintWriter(getResultOutput(result));
				writer.print(content);
				writer.close();

				// Set its size:
				result.setSize(content.getBytes("UTF-8").length);

				// Add the result into the job:
				job.addResult(result);

			}catch(IOException e){
				// If there is an error, encapsulate it in an UWSException so that an error summary can be published:
				throw new UWSException(UWSException.INTERNAL_SERVER_ERROR, e, "Impossible to write the result of the job " + job.getJobId() + "!", ErrorType.TRANSIENT);
			}

		}
	}

}
